@@ -27,7 +27,8 @@ from subprocess import PIPE,STDOUT
 from shutil import copy2
 from calculate.core.server.func import safetyWrapper
 from calculate.lib.utils.files import (runOsCommand,pathJoin,scanDirectory,
-                                       process,listDirectory,STDOUT,checkUtils)
+                                       isMount,process,listDirectory,STDOUT,
+                                       checkUtils)
 from calculate.lib.utils.common import (appendProgramToEnvFile,
                         removeProgramToEnvFile, getTupleVersion,
                         cmpVersion,getUserPassword,
@@ -154,7 +155,8 @@ class Install(color_print):
         """Check that system has grub2 in current and installed system"""
         if self.clVars.Get('os_grub2_path'):
             return bool(
-                   filter(lambda x:x.startswith('grub-1.99'),
+                   filter(lambda x:x.startswith('grub-1.99') or \
+                                   x.startswith('grub-2'),
                    listDirectory('/var/db/pkg/sys-boot')))
         return False
 
@@ -249,90 +251,6 @@ class Install(color_print):
         else:
             return dirsFiles
 
-    def checkDuplicate(self,datalist,name,key=lambda x:x):
-        """Check on duplicate and print error"""
-        keydata = map(key,datalist)
-        dupKey = set(filter(lambda x:keydata.count(x)>1, keydata))
-        if dupKey:
-            self.printERROR(_("Duplicated {keyname}: {keylist}").format(
-                                keyname=name,keylist=",".join(dupKey)))
-            return False
-        return True
-
-    def setVarList(self,varsList,matrix):
-        """Set matrix to vars"""
-        if not matrix:
-            map(lambda x:self.clVars.Set(x,[],True),varsList)
-        else:
-            map(lambda x:self.clVars.Set(x[0],list(x[1]),True),
-            zip(varsList,zip(*matrix)))
-
-    def setDisks(self):
-        """
-        TODO: remove check grub2_path in disk variables
-        """
-        try:
-            if not self.clVars.Get('os_grub2_path'):
-                self.checkForLegacyGrub()
-            else:
-                self.checkForGrub2()
-        except InstallError,e:
-            error.append(e)
-        return True
-
-    def setUsers(self,listUsers,autologinUser):
-        """Set users data (migration)"""
-        if autologinUser == "none":
-            autologinUser = ""
-        if autologinUser or autologinUser == "":
-            self.clVars.Set('cl_autologin', autologinUser, force=True)
-        else:
-            autologinUser = self.clVars.Get('cl_autologin')
-        autologinUser = [autologinUser] if autologinUser else []
-        listUsers = listUsers or []
-        if listUsers or autologinUser:
-            migrateUsers = list(set(listUsers+autologinUser))
-            migrateUsers.sort()
-            migrateUsers = ["root"] + filter(lambda x: x!="root", migrateUsers)
-            self.clVars.Set('cl_migrate_user', migrateUsers, force=True)
-        return True
-
-
-    def setInstallOptions(self, listDisks, listBinds, listSwaps, listUsers,
-                          autologinUser, brDisk):
-        """Set install options (set users, disks and boot record"""
-        try:
-            if self.setUsers(listUsers,autologinUser) and \
-                self.setDisks(listDisks,listBinds,listSwaps) and \
-                self.setBR(brDisk):
-                return self.createListOptions()
-            else:
-                return False
-        except InstallError,e:
-            self.printERROR(str(e).strip())
-            return False
-
-    def getDeviceByField(self,field,value, secondPrefix="os_disk"):
-        """Get device by fields (install load format uuid grub part name)"""
-        return self.getFieldByField('dev',field,value,
-                                    secondPrefix=secondPrefix)
-
-    def getFieldByField(self,resField,field,value, firstPrefix="os_disk",
-                        secondPrefix="os_disk"):
-        res = filter(lambda x: x[1] == value,
-                zip(self.clVars.Get('%s_%s'%(firstPrefix,resField)),
-                    self.clVars.Get('%s_%s'%(secondPrefix,field)))) or\
-                    [("","")]
-        return res[0][0]
-
-    def getFieldByDevice(self,field,device):
-        """Get value of field by device"""
-        device_hash = self.clVars.Get('os_disk_hash')
-        if device in device_hash and field in device_hash[device]:
-            return device_hash[device][field]
-        else:
-            return ""
-
     def setActivePartition(self,partition):
         """Change partition id, specified by systemid,set active"""
         deviceName = detectDeviceForPartition(partition)
@@ -454,29 +372,62 @@ class Install(color_print):
         """
         Install grub2 boot loader
         """
-        for bootPath in ("/boot","/"):
-            bootDisk = self.clVars.Select("os_install_disk_dev",
-                where="os_install_disk_mount",eq=bootPath,limit=1)
-            if bootDisk:
-                self.setActivePartition(bootDisk)
-                break
         cmdGrubInstall = self.clVars.Get('os_grub2_path')
         if not cmdGrubInstall:
             raise DistributiveError(_("Failed to install the bootloader"))
-
-        for mbrDisk in self.clVars.Get('os_install_mbr'):
-            process("sync").success()
-            if self.clVars.Get('os_install_scratch') == "on" and \
-                self.clVars.Get('cl_action') != "system":
-                prefixBoot = "/mnt/scratch"
-            else:
-                prefixBoot = "/"
-            grubProcess = process(cmdGrubInstall,
-                              "--boot-directory=%s"%pathJoin(prefixBoot,
+        process("sync").success()
+        if self.clVars.Get('os_install_scratch') == "on" and \
+            self.clVars.Get('cl_action') != "system":
+            prefixBoot = "/mnt/scratch"
+        else:
+            prefixBoot = "/"
+        if self.clVars.GetBool('os_install_uefi_set'):
+            grubParams = ["--boot-directory=%s"%pathJoin(prefixBoot,
                               target.getBootDirectory()),
-                              mbrDisk, "-f", stderr=STDOUT,envdict=os.environ)
+                              "--target=x86_64-efi",
+                              "--efi-directory=%s"%
+                              target.getEfiDirectory(),
+                              "-f"]
+            if self.clVars.Get('os_install_root_type') == 'usb-hdd':
+                grubParams.append("--removable")
+            if self.clVars.Get('cl_action') != "system" and \
+                not isMount('/boot/efi'):
+                raise DistributiveError(_("Failed to install the bootloader. "
+                                          "/boot/efi is not mounted."))
+            grubProcess = process(cmdGrubInstall,
+                              *grubParams, stderr=STDOUT,envdict=os.environ)
             if grubProcess.failed():
                 raise DistributiveError(_("Failed to install the bootloader"))
+            efiBootMgr = varsShare().getProgPath('/usr/sbin/efibootmgr')
+            dmesg = varsShare().getProgPath('/bin/dmesg')
+            if efiBootMgr and dmesg:
+                if not re.search('Boot.*calculate',
+                        process(efiBootMgr).read(),re.M) and \
+                   re.search('efivars.*set_variable.*failed',
+                        process(dmesg).read(),re.M):
+                    raise DistributiveError(
+                        _("Failed to create the UEFI boot record"))
+        else:
+            for bootPath in ("/boot","/"):
+                bootDisk = self.clVars.Select("os_install_disk_dev",
+                    where="os_install_disk_mount",eq=bootPath,limit=1)
+                if bootDisk:
+                    self.setActivePartition(bootDisk)
+                    break
+
+            if filter(lambda x:"2.00" in x,
+                           process(cmdGrubInstall,'-v')):
+                platform = ["--target=i386-pc"]
+            else:
+                platform = []
+            for mbrDisk in self.clVars.Get('os_install_mbr'):
+                grubProcess = process(cmdGrubInstall,
+                                  "--boot-directory=%s"%pathJoin(prefixBoot,
+                                  target.getBootDirectory()),
+                                  mbrDisk, "-f", *platform,
+                                  stderr=STDOUT,envdict=os.environ)
+                if grubProcess.failed():
+                    raise DistributiveError(_("Failed to install the bootloader"))
 
     def installLegacyGrubBootloader(self,target):
         """
@@ -590,16 +541,14 @@ class Install(color_print):
         self.endTask()
 
         self.startTask(_("Copying configuration files to the new system"))
-        fileMask = r"/etc/udev/rules\.d/70-persistent-net\.rules"
         if self.clVars.Get('os_root_type') != "livecd":
             fileMask = \
-                "(%s|/etc/ssh/ssh_host_.*|/root/.ssh/(id_.*|known_hosts))"% \
-                    fileMask
-        fileCpy = otherfilesCopy(target=targetDistr.getDirectory(),
-                                 reTest=fileMask)
-        fileCpy.performCopy('/etc')
-        if path.exists('/root/.ssh'):
-            fileCpy.performCopy('/root/.ssh')
+                "(/etc/ssh/ssh_host_.*|/root/.ssh/(id_.*|known_hosts))"
+            fileCpy = otherfilesCopy(target=targetDistr.getDirectory(),
+                                     reTest=fileMask)
+            fileCpy.performCopy('/etc')
+            if path.exists('/root/.ssh'):
+                fileCpy.performCopy('/root/.ssh')
         self.endTask()
 
         # optimize initrd
@@ -681,7 +630,8 @@ class Install(color_print):
         if self.clVars.Get('cl_setup') == 'boot' and \
             self.clVars.Get('os_root_type') != 'livecd':
             target = self.clVars.Get('cl_image')
-            if self.clVars.Get('os_install_mbr'):
+            if self.clVars.Get('os_install_mbr') or \
+               self.clVars.Get('os_install_uefi_set') == 'on':
                 self.startTask(_("Installing the bootloader"))
                 self.prepareBoot(target)
                 self.endTask()
@@ -743,7 +693,8 @@ class Install(color_print):
                 devices,
                 self.clVars.Get('cl_autopartition_disk_data'),
                 self.clVars.GetBool('cl_autopartition_lvm_set'),
-                self.clVars.Get('cl_autopartition_lvm_vgname'))
+                self.clVars.Get('cl_autopartition_lvm_vgname'),
+                self.clVars.Get('cl_autopartition_bios_grub_set')=="on")
             self.endTask()
         self.targetDistr = self.clVars.Get('cl_target')
         self.sourceDistr = self.clVars.Get('cl_image')
@@ -787,11 +738,17 @@ class Install(color_print):
             self.endTask()
             self.closeClTemplate()
         # change boot config
-        if self.clVars.Get('os_install_mbr') and \
+        if (self.clVars.Get('os_install_mbr') or
+            self.clVars.Get('os_install_uefi_set') == 'on') and \
             self.clVars.Get('os_install_pxe') == "off":
             self.startTask(_("Preparing the system for reboot"))
             self.prepareBoot(self.targetDistr)
         self.endTask()
+        if self.askConfirm(_("Would you like to reboot your computer "
+                             "now to complete the installation?"),
+                                default="no") == "yes":
+            os.system("/sbin/reboot")
+            return True
         return True
 
     def rndString(self):
