@@ -472,6 +472,8 @@ class migrate(color_print):
         """Migrate users ang groups to new system"""
         if not self.checkPermFiles():
             return False
+        if not any(addUsersList):
+            addUsersList = []
         migrateUsers = ["root"]+\
                        map(lambda x: x[0], addUsersList + pwdUsersList)
         for existMigrUser in existsMigrateUsers:
