@@ -103,3 +103,13 @@ class VariableAcInstallUnmerge(ActionVariable):
             self.Get('cl_ebuild_phase') in ('prerm','postrm'):
             return "on"
         return "off"
+
+class VariableAcInstallPatch(ActionVariable):
+    """
+    Action variable which has value "on" 
+    in ebuild phase preinst or reconfigure system
+    """
+    def action(self,cl_action):
+        if cl_action in ("patch",):
+            return "on"
+        return "off"
