@@ -24,7 +24,7 @@ from calculate.lib.datavars import TableVariable,Variable,VariableError, \
 from calculate.lib.utils.device import (getUdevDeviceInfo,getDeviceType,
                         getPartitionType,getPartitionDevice,getRaidPartitions,
                         getLvmPartitions,getPartitionSize,getUUIDDict,
-                        humanreadableSize)
+                        humanreadableSize,refreshUdev)
 from calculate.lib.utils.files import (listDirectory,pathJoin,readFile,FStab,
                                         readLinesFile,process,getProgPath)
 from calculate.install.cl_distr import PartitionDistributive
@@ -66,24 +66,34 @@ class AutoPartition:
     """
     Autopartition maker
     """
-    def recreateSpace(self,table,device,data,lvm,vgname):
+    def recreateSpace(self,table,device,data,lvm,vgname,bios_grub):
+        """
+        Recreate disk space by table device data lvm flag and vgname
+        
+        table (gpt,dos)
+        device - list devices ['/dev/sda']
+        data - partitions data [['/dev/calculate/boot','/boot','ext4',534234234'gpt]...]
+        lvm True or False
+        vgname lvm Volume Group Name
+        """
         if lvm:
-            self.recreateLvm(table,device,data,vgname)
+            self.recreateLvm(table,device,data,vgname,bios_grub)
         else:
-            self.recreatePartitionTable(table,device,data)
+            self.recreatePartitionTable(table,device,data,bios_grub)
+        refreshUdev()
 
-    def recreatePartitionTable(self,table,device,data):
+    def recreatePartitionTable(self,table,device,data,bios_grub):
         """
         """
         mapDispatch = {'dos':self.recreatePartitionTableDos,
                        'gpt':self.recreatePartitionTableGpt}
         if table in mapDispatch:
-            mapDispatch[table](device,data)
+            mapDispatch[table](device,data,bios_grub)
         else:
             raise AutoPartitionError(
                     _('Autopartitioning for %s is not supported')%table)
 
-    def recreatePartitionTableDos(self,device,data):
+    def recreatePartitionTableDos(self,device,data,bios_grub):
         """
         Create DOS partition table by /sbin/fdisk
         """
@@ -115,7 +125,7 @@ class AutoPartition:
         fdisk.success()
         self._waitDevice(device[0]+str(num-1))
 
-    def recreatePartitionTableGpt(self,device,data):
+    def recreatePartitionTableGpt(self,device,data,bios_grub=True):
         """
         Create GPT partition table by /sbin/gdisk
         """
@@ -125,23 +135,27 @@ class AutoPartition:
         MAX_SIZE = "\n\n"
         WRITE_AND_QUIT = "w\ny\n"
 
+        BIOS_BOOT_PART_NUM = 4
+        BIOS_BOOT_PART_SIZE = "2M"
+
         fdiskProg = getProgPath('/usr/sbin/gdisk')
         fdisk = process(fdiskProg,device[0])
         fdisk.write(NEW_PARTITION_TABLE)
         num = 1
-        biosBootCreated = False
+        biosBootCreated = not bios_grub
         for size in map(lambda x:str(Sizes().to_K(int(x))) \
                                  if x.isdigit() else x,
                     map(operator.itemgetter(3),data)):
-            if num == 4:
-                fdisk.write(NEW_BIOSBOOT_PARTITION%"+2M")
+            if num == BIOS_BOOT_PART_NUM and not biosBootCreated:
+                fdisk.write(NEW_BIOSBOOT_PARTITION%
+                            ("+%s"%BIOS_BOOT_PART_SIZE))
                 biosBootCreated = True
                 num += 1
             if size == "allfree":
                 if biosBootCreated:
                     size = MAX_SIZE
                 else:
-                    size = "-2M\n\n"
+                    size = "-%s\n\n"%BIOS_BOOT_PART_SIZE
             else:
                 size = "+%sK\n\n"%size
             fdisk.write(NEW_PARTITION+size)
@@ -237,14 +251,23 @@ class AutoPartition:
                                   '--zero-superblock',disk_dev)
         return not failed
 
-    def recreateLvm(self,table,devices,data,vgname):
+    def recreateLvm(self,table,devices,data,vgname,bios_grub):
         """
         Create GPT partition table by /sbin/gdisk
         """
-        for device in devices:
-            self.recreatePartitionTable(table,[device],[['','','','allfree']])
+        for i,device in enumerate(devices):
+            if i == 0:
+                DEV,MP,FS,SIZE,TABLE=0,1,2,3,4
+                notInLvm = filter(lambda x:"boot" in x[MP],data)
+                self.recreatePartitionTable(table,
+                    [device],notInLvm+[['','','','allfree']],bios_grub)
+            else:
+                self.recreatePartitionTable(table,
+                    [device],[['','','','allfree']],bios_grub)
 
-        disks = map(lambda x:"%s1"%x,devices)
+        lvmPartOffset = 1 + len(notInLvm)
+        iPart = [lvmPartOffset]+[1]*(len(devices)-1)
+        disks = map(lambda x:"%s%d"%x,zip(devices,iPart))
         if not self._createPhisicalVolumes(disks):
             raise AutopartitionError(
                 _("Failed to create physical volumes from %s")
@@ -257,7 +280,8 @@ class AutoPartition:
                         disks=",".join(devices)))
 
         lvCreateProg = getProgPath('/sbin/lvcreate')
-        for disk,size in map(operator.itemgetter(0,3),data):
+        for disk,size in map(operator.itemgetter(0,3),
+                         filter(lambda x:not "boot" in x[MP],data)):
             lvname = disk.rpartition('/')[2]
             if not self._createLogicalVolume(vgname,lvname,size):
                 raise AutopartitionError(
@@ -277,7 +301,7 @@ class AutopartitionHelper:
         else:
             beforeVal = ""
         for line in listvalue:
-            if line in ("home","lvm","raid"):
+            if line in ("home","lvm","raid","grub"):
                 continue
             if line == beforeVal:
                 yield "root2"
@@ -294,6 +318,7 @@ class AutopartitionHelper:
                  'boot':'/boot',
                  'root2':'/',
                  'root':'',
+                 'uefi':'/boot/efi',
                  'data':'/var/calculate'}
         return mapMp.get(value,'')
 
@@ -306,11 +331,12 @@ class AutopartitionHelper:
         return mapMp.get(value,'')
 
     def getAutopartitionScheme(self):
-        optOrder = {'boot':0,
-                    'swap':1,
-                    'root':2,
-                    'data':3,
-                    'home':4}
+        optOrder = {'uefi':0,
+                    'boot':1,
+                    'swap':2,
+                    'root':3,
+                    'data':4,
+                    'home':5}
         return sorted(self.Get('cl_autopartition_scheme'),key=optOrder.get)
 
     def uncompatible(self):
@@ -459,15 +485,23 @@ class VariableClAutopartitionScheme(AutopartitionHelper,Variable,AutoPartition):
     Autopartition scheme
     """
     type = "choice-list"
-    value = ["swap","root","data"]
     element = "selecttable"
     opt = ["--auto-scheme"]
     metavalue = "AUTOPARTOPTS"
+    check_after = ["cl_autopartition_table"]
 
     def init(self):
         self.help = _("autopartition options")
         self.label = _("Autopartition options")
 
+    def get(self):
+        if self.Get('os_uefi_set') == 'on':
+            return ["uefi","swap","root","data","home"]
+        elif self.Get('cl_autopartition_table') == 'gpt':
+            return ["swap","root","data","home","bios"]
+        else:
+            return ["swap","root","data","home"]
+
     def choice(self):
         return [
                 ("swap",_("Swap partition")),
@@ -475,9 +509,33 @@ class VariableClAutopartitionScheme(AutopartitionHelper,Variable,AutoPartition):
                 ("data",_("/var/calculate partition")),
                 ("home",_("Mount /var/calculate/home to /home")),
                 ("boot",_("Separate boot partition")),
-                ("lvm",_("Use LVM"))
+                ("uefi",_("Use UEFI bootloader")),
+                ("lvm",_("Use LVM")),
+                ("grub",_("Create bios_grub partition")),
                ]
 
+    def check(self,value):
+        if "uefi" in value:
+            if self.Get('os_uefi_set') == 'off':
+                raise VariableError(
+                  _("System must be loaded in UEFI for using this bootloader"))
+            if self.Get('os_install_arch_machine') != 'x86_64':
+                raise VariableError(
+                    _("Architecture of the target system must be x86_64 "
+                      "for using UEFI bootloader"))
+            if self.Get('cl_autopartition_table') != 'gpt':
+                raise VariableError(
+                    _("Partition table must be GPT for using UEFI bootloader"))
+        else:
+            if self.Get('cl_autopartition_table') == 'gpt' and not "grub" in value:
+                raise VariableError(
+                    _("For grub installation need bios_grub partition"))
+        if "grub" in value:
+            if self.Get('cl_autopartition_table') != 'gpt':
+                raise VariableError(
+                    _("Partition table must be GPT for bios_grub partition"))
+
+
 class VariableClAutopartitionRootSize(AutopartitionHelper,Variable):
     """
     Root partition size for autopartition
@@ -555,6 +613,16 @@ class VariableClAutopartitionDiskData(ReadonlyTableVariable):
               'cl_autopartition_disk_size',
               'cl_autopartition_disk_part']
 
+class VariableClAutopartitionBiosGrubSet(ReadonlyVariable):
+    """
+    Create bios_grub partition
+    """
+    type = "bool"
+
+    def get(self):
+        return "on" if "grub" in self.Get('cl_autopartition_scheme') else "off"
+
+
 class VariableClAutopartitionLvmSet(ReadonlyVariable):
     """
     Using LVM for autopartition
@@ -564,6 +632,16 @@ class VariableClAutopartitionLvmSet(ReadonlyVariable):
     def get(self):
         return "on" if "lvm" in self.Get('cl_autopartition_scheme') else "off"
 
+class VariableClAutopartitionUefiSet(ReadonlyVariable):
+    """
+    Using UEFI bootloader
+    """
+    type = "bool"
+
+    def get(self):
+        return "on" if "uefi" in self.Get('cl_autopartition_scheme') else "off"
+
+
 class VariableClAutopartitionLvmVgname(Variable):
     """
     Volume group name for LVM autopartition
@@ -595,17 +673,27 @@ class VariableClAutopartitionDiskDev(ReadonlyVariable,AutopartitionHelper):
         for line in self.deviceOpts(scheme):
             # for dos 4 - extension
             # for gpt 4 - for bios_boot
-            if number == 4:
+            if number == 4 and (
+                self.Get('cl_autopartition_bios_grub_set')=='on' or
+                dos):
                 number += 1
             yield "%s%d"%(devices[0],number)
             number += 1
 
     def generateLvm(self,scheme=[],devices=[],**kwargs):
         vgname = self.Get('cl_autopartition_lvm_vgname')
+        number = 1
         for line in map(lambda x:{'root':'root2',
                                   'root2':'root'}.get(x,x),
                     self.deviceOpts(scheme)):
-            yield "/dev/%s/%s"%(vgname,line)
+            if line in ("boot","uefi"):
+                if number == 4 and \
+                    self.Get('cl_autopartition_bios_grub_set')=='on':
+                    number += 1
+                yield "%s%d"%(devices[0],number)
+                number += 1
+            else:
+                yield "/dev/%s/%s"%(vgname,line)
 
     def diskGenerator(self):
         if self.Get('cl_autopartition_lvm_set') == 'on':
@@ -658,6 +746,7 @@ class VariableClAutopartitionDiskFormat(ReadonlyVariable,AutopartitionHelper):
         """
         for line in self.deviceOpts(scheme):
             if line == "swap": yield "swap"
+            elif line == "uefi": yield "vfat"
             else: yield FileSystemManager.defaultFS['hdd']
 
     def get(self):
@@ -717,11 +806,12 @@ class VariableClAutopartitionDiskSize(ReadonlyVariable,AutopartitionHelper):
     """
     type = "list"
 
-    def generateSize(self,scheme,memory,bootsize,rootsize,availsize):
+    def generateSize(self,scheme,memory,bootsize,uefisize,rootsize,availsize):
         args = {'swap':memory,
                 'root': rootsize,
                 'root2': rootsize,
                 'boot': bootsize,
+                'uefi': uefisize,
                }
         deviceOpts = list(self.deviceOpts(scheme))
         for line in deviceOpts[:-1]:
@@ -743,6 +833,7 @@ class VariableClAutopartitionDiskSize(ReadonlyVariable,AutopartitionHelper):
                 return list(self.generateSize(scheme,
                            self.Get('cl_autopartition_swap_size'),
                            self.Get('cl_autopartition_boot_size'),
+                           self.Get('cl_autopartition_uefi_size'),
                            str(int(self.Get('cl_autopartition_root_size'))
                                    *Sizes.M),int(availSize)))
         return []
@@ -753,6 +844,12 @@ class VariableClAutopartitionDiskSize(ReadonlyVariable,AutopartitionHelper):
                map(lambda x:allSize if x == "allfree" else x,
                self.Get()))
 
+class VariableClAutopartitionUefiSize(Variable):
+    """
+    Size of EF00 partition
+    """
+    value = str(200*Sizes.M)
+
 class VariableClAutopartitionBootSize(Variable):
     """
     Size of boot partition
