@@ -338,7 +338,7 @@ class VariableOsDiskMount(ReadonlyVariable,DeviceHelper):
 
     def get(self):
         disk_hash = self.Get('os_disk_dev')
-        fstab = FStab('/etc/fstab')
+        fstab = FStab('/etc/fstab',devs=disk_hash)
         rootdev = self.Get('os_root_dev')
         return map(lambda x: '/' if x == rootdev else fstab.getBy(eq=x) or "",
                self.Get('os_disk_dev'))
@@ -364,7 +364,7 @@ class VariableOsDiskFormat(ReadonlyVariable):
 
     def get(self):
         """Get current disk filesystem"""
-        fstab = FStab('/etc/fstab')
+        fstab = FStab('/etc/fstab',devs=self.Get('os_disk_dev'))
         def getFormat(device):
             prop = getUdevDeviceInfo(name=device)
             return prop.get('FSTAB_TYPE') or \
@@ -470,7 +470,9 @@ class VariableOsDiskId(ReadonlyVariable,DeviceHelper):
         mapTypeUUID = {'ebd0a0a2-b9e5-4433-87c0-68b6b72699c7':'0700',
                        '0657fd6d-a4ab-43c4-84e5-0933c84b4f4f':'8200',
                        'a19d880f-05fc-4d3b-a006-743f0f84911e':'FD00',
-                       '21686148-6449-6e6f-744e-656564454649':'EF02'}
+                       '21686148-6449-6e6f-744e-656564454649':'EF02',
+                       'c12a7328-f81f-11d2-ba4b-00a0c93ec93b':'EF00',
+                       '0fc63daf-8483-4772-8e79-3d69d8477de4':'8300'}
         return map(lambda x:mapTypeUUID.get(x,x),
                map(lambda x:x.rpartition("x")[2],
                self.mapUdevProperty('os_disk_dev','ID_PART_ENTRY_TYPE','')))
@@ -543,7 +545,7 @@ class VariableOsDiskOptions(ReadonlyVariable):
     type = "list"
 
     def get(self):
-        fstab = FStab('/etc/fstab')
+        fstab = FStab('/etc/fstab',devs=self.Get('os_disk_dev'))
         def getFormat(device):
             return fstab.getBy(what=fstab.OPTS,eq=device)
         return map(getFormat,
@@ -566,7 +568,7 @@ class VariableOsBindPath(ReadonlyVariable):
     type = "list"
 
     def get(self):
-        fstab = FStab('/etc/fstab')
+        fstab = FStab('/etc/fstab',devs=self.Get('os_disk_dev'))
         return fstab.getBy(what=fstab.NAME,where=fstab.OPTS,
                 _in="bind",allentry=True)
 
@@ -577,7 +579,7 @@ class VariableOsBindMountpoint(ReadonlyVariable):
     type = "list"
 
     def get(self):
-        fstab = FStab('/etc/fstab')
+        fstab = FStab('/etc/fstab',devs=self.Get('os_disk_dev'))
         return fstab.getBy(what=fstab.DIR,where=fstab.OPTS,
                 _in="bind",allentry=True)
 
@@ -598,6 +600,38 @@ class LocationHelper:
                 _("The layout is not available with autopartitioning")
         return ""
 
+class VariableOsLocationBriefData(LocationHelper,TableVariable):
+    source = ["os_location_source",
+              "os_location_dest",
+              "os_location_format",
+              "os_location_perform_format",
+              "os_location_size"]
+
+    orig_source = [('os_install_disk_dev',
+                   'os_install_disk_mount',
+                   'os_install_disk_format',
+                   'os_install_disk_perform_format',
+                   'os_install_disk_size'),
+                   ('os_install_bind_path',
+                    'os_install_bind_mountpoint','','','')]
+
+    def init(self):
+        self.label = _("Layout")
+
+    def get(self,hr=False):
+        for varname,value in ifilter(lambda x:type(x[1]) != list,
+                             imap(lambda x:(x,self.Get(x)),
+                             ifilter(None,
+                             chain(*self.orig_source)))):
+            raise VariableError(
+                _("Source variable %s does not contain a list")%varname)
+        return list(
+               chain(*
+               map(lambda k:map(list,
+                            izip_longest(
+                            *map(lambda x:self.Get(x,humanreadable=hr) if x else '',
+                            k),fillvalue='')),self.orig_source))) or [[]] 
+
 class VariableOsLocationData(LocationHelper,TableVariable):
     """
     Select installation disk variable
@@ -662,7 +696,8 @@ class VariableOsLocationSource(LocationHelper,DeviceHelper,Variable):
 
     def get(self):
         if self.Get('cl_autopartition_set') == "on":
-            return self.Get('cl_autopartition_disk_dev') + \
+            return map(getCommonDeviceName,
+                   self.Get('cl_autopartition_disk_dev')) + \
                    self.Get('cl_autopartition_bind_path')
         else:
             return self.availDevs()
@@ -769,13 +804,18 @@ class VariableOsLocationDest(LocationHelper,Variable):
             if type(val) == str and val:
                 return path.normpath(val)
             return val
-        value = map(normpath,value)
+        def eficonvert(val):
+            if val.lower() in ("efi","uefi"):
+                return "/boot/efi"
+            else:
+                return val
+        value = map(normpath,map(eficonvert,value))
         return map(lambda x:x or "/",value)
 
     def choice(self):
         if self.Get('cl_install_type') == 'flash': return ["/",""]
         else: return ['/','/boot','/var/calculate','/home',
-                      '/usr','/var','/tmp','swap','']
+                      '/usr','/var','/tmp','swap','/boot/efi','']
 
     def check(self,value):
         """Check set location source"""
@@ -860,6 +900,17 @@ class VariableOsLocationDest(LocationHelper,Variable):
         if cdromPartitions:
             raise VariableError(_("Unable to use CDROM %s for installation")%
                               cdromPartitions)
+        #############################
+        # detect right part for uefi#
+        #############################
+        efiDisks = map(lambda x:getCommonDeviceName(x[0]),
+                   filter(lambda x:x[1] == '/boot/efi',
+                       zip(source,value)))
+        wrongPart = self.Select('os_install_disk_dev',
+                               where='os_install_disk_type',
+                               ne='disk-partition')
+        if set(efiDisks) & set(wrongPart):
+            raise VariableError(_("UEFI partition must be partition on disk"))
         ###############################
         # check cross bind mount points
         ###############################
@@ -936,10 +987,12 @@ class VariableOsLocationFormat(LocationHelper,Variable):
             if mount and not fs and dev.startswith('/dev/'):
                 if mount == "swap":
                     return "swap"
+                elif mount == '/boot/efi':
+                    return "vfat"
                 if dev in diskFormat and diskFormat[dev] in availFS:
                     if mount.count('/') == 1:
-                        if FileSystemManager.checkFSForType(diskFormat[dev],
-                                                     osInstallRootType):
+                        if FileSystemManager.checkFSForTypeMount(diskFormat[dev],
+                                                     osInstallRootType,mount):
                             return diskFormat[dev]
                     else:
                         return diskFormat[dev]
@@ -949,6 +1002,7 @@ class VariableOsLocationFormat(LocationHelper,Variable):
         return wrap
 
     def set(self,value):
+        value = map(lambda x:"vfat" if x == "uefi" else x,value)
         mount = self.Get("os_location_dest")
         source = self.Get("os_location_source")
         return map(self.defaultFormat(),
@@ -963,10 +1017,10 @@ class VariableOsLocationFormat(LocationHelper,Variable):
                 raise VariableError(
                     _("The bind mount point does not use filesystem"))
             # check compatible fs for mount point only root dirs
-            if dev.startswith('/dev/') and mp and (mp.count('/') == 1 or
-                        mp == '/var/calculate'):
-                if not FileSystemManager.checkFSForType(fs,
-                                                        osInstallRootType):
+            if dev.startswith('/dev/') and mp and (mp.count('/') == 1 or 
+                        mp in ('/var/calculate','/boot/efi')):
+                if not FileSystemManager.checkFSForTypeMount(fs,
+                                                        osInstallRootType,mp):
                     raise VariableError(
                         _("The filesystem for '%(mp)s' should not be '%(opt)s'")
                         %{'mp':mp, 'opt':fs}+" "+
@@ -1130,7 +1184,6 @@ class VariableOsLocationSize(LocationHelper,SourceReadonlyVariable):
                map(getCommonDeviceName,
                self.Get(self.indexField))))
 
-
 class VariableClUuidSet(Variable):
     """
     Use or not UUID for /etc/fstab
@@ -1172,9 +1225,69 @@ class VariableOsInstallDiskData(ReadonlyTableVariable):
               "os_install_disk_part",
               "os_install_disk_parent"]
 
-class VariableOsInstallDiskDev(ReadonlyVariable):
+class VariableOsInstallDiskEfi(ReadonlyVariable):
+    type = "list"
+
+    def get(self):
+        if self.Get('os_install_root_type') == 'usb-hdd':
+            validParent = self.Select('os_install_disk_parent_base',
+                       where='os_install_disk_mount_base',eq='/')
+        else:
+            validParent = self.Select('os_device_dev',
+                       where='os_device_type',eq='hdd')
+        efiPartitions = self.Select('os_disk_dev',
+                where=['os_disk_id','os_disk_parent'],
+                func=lambda x:x[0] == 'EF00' and x[1] in validParent)
+        return list(set(efiPartitions)-set(self.Get('os_location_source')))
+
+class VariableOsAddonDiskDev(ReadonlyVariable,DeviceHelper):
+    type = "list"
+    def get(self):
+        if self.Get('os_install_uefi_set') == 'on':
+            if not "/boot/efi" in self.Get('os_location_dest'):
+                efiPartition = self.Get('os_install_disk_efi')
+                if efiPartition:
+                    return efiPartition[:1]
+        return []
+
+class VariableOsAddonDiskMount(ReadonlyVariable,DeviceHelper):
+    type = "list"
+    def get(self):
+        if self.Get('os_install_uefi_set') == 'on':
+            if not "/boot/efi" in self.Get('os_location_dest'):
+                efiPartition = self.Get('os_install_disk_efi')
+                if efiPartition:
+                    return ['/boot/efi']
+        return []
+
+class VariableOsAddonDiskFormat(ReadonlyVariable,DeviceHelper):
+    type = "list"
+    def get(self):
+        if self.Get('os_install_uefi_set') == 'on':
+            if not "/boot/efi" in self.Get('os_location_dest'):
+                efiPartition = self.Get('os_install_disk_efi')
+                if efiPartition:
+                    return ['vfat']
+        return []
+
+class VariableOsAddonDiskPerformFormat(ReadonlyVariable,DeviceHelper):
+    type = "list"
+    def get(self):
+        if self.Get('os_install_uefi_set') == 'on':
+            if not "/boot/efi" in self.Get('os_location_dest'):
+                efiPartition = self.Get('os_install_disk_efi')
+                if efiPartition:
+                    fsPart = self.Select('os_disk_format',where='os_disk_dev',
+                                         eq=efiPartition[0],limit=1)
+                    if fsPart != "vfat":
+                        return ['on']
+                    else:
+                        return ['off']
+        return []
+
+class VariableOsInstallDiskDevBase(ReadonlyVariable,DeviceHelper):
     """
-    Disks for installation
+    Variable using for resolv cyclic deps
     """
     type = "list"
 
@@ -1185,6 +1298,38 @@ class VariableOsInstallDiskDev(ReadonlyVariable):
                zip(self.Get('os_location_source'),
                    self.Get('os_location_dest')))))
 
+class VariableOsInstallDiskParentBase(SourceReadonlyVariable):
+    """
+    Partition parent devices using for install
+    """
+    type = "list"
+    indexField = "os_install_disk_dev_base"
+
+    def getMap(self):
+        diskParent = dict(self.ZipVars('os_disk_dev','os_disk_parent'))
+        # replace value for autopartition
+        if self.Get('cl_autopartition_set') == 'on':
+            diskParent.update(zip(self.Get('cl_autopartition_disk_dev'),
+                                  self.Get('cl_autopartition_device')*
+                                  len(self.Get('cl_autopartition_disk_dev'))))
+        return diskParent
+
+    humanReadable = Variable.humanReadable
+
+class VariableOsInstallDiskDev(ReadonlyVariable,DeviceHelper):
+    """
+    Disks for installation
+    """
+    type = "list"
+
+    def get(self):
+        return self.Get('os_addon_disk_dev')+\
+               self.Get('os_install_disk_dev_base')
+
+    def humanReadable(self):
+        return map(lambda x:self.getPerfectName(x,defaultValue=x),
+               self.Get())
+
 class VariableOsInstallDiskUuid(ReadonlyVariable):
     """
     Uudi for install
@@ -1196,6 +1341,20 @@ class VariableOsInstallDiskUuid(ReadonlyVariable):
         hashUUID = getUUIDDict(revers=True)
         return map(lambda x:hashUUID.get(x,"")[5:],diskDev)
 
+class VariableOsInstallDiskMountBase(ReadonlyVariable):
+    """
+    List mounted points for installed system
+
+    Variable use for resolv cyclic deps by UEFI vars
+    """
+    type = "list"
+
+    def get(self):
+        return map(lambda x:x[1],
+               filter(lambda x:x[0].startswith('/dev/') and x[1],
+               zip(self.Get('os_location_source'),
+                   self.Get('os_location_dest'))))
+
 class VariableOsInstallDiskMount(ReadonlyVariable):
     """
     List mounted points for installed system
@@ -1204,10 +1363,8 @@ class VariableOsInstallDiskMount(ReadonlyVariable):
 
     def get(self):
         """Get install disk dest"""
-        return map(lambda x:x[1],
-               filter(lambda x:x[0].startswith('/dev/') and x[1],
-               zip(self.Get('os_location_source'),
-                   self.Get('os_location_dest'))))
+        return self.Get('os_addon_disk_mount')+\
+               self.Get('os_install_disk_mount_base')
 
 class VariableOsInstallDiskUse(ReadonlyVariable):
     """
@@ -1247,11 +1404,12 @@ class VariableOsInstallDiskFormat(ReadonlyVariable):
     type = "choice-list"
 
     def get(self):
-        return map(lambda x:x[2],
+        _format = map(lambda x:x[2],
                filter(lambda x:x[0].startswith('/dev/') and x[1],
                zip(self.Get('os_location_source'),
                    self.Get('os_location_dest'),
                    self.Get('os_location_format'))))
+        return self.Get('os_addon_disk_format')+_format
 
 class VariableOsInstallDiskPerformFormat(ReadonlyVariable):
     """
@@ -1260,11 +1418,12 @@ class VariableOsInstallDiskPerformFormat(ReadonlyVariable):
     type = "bool-list"
 
     def get(self):
-        return map(lambda x:x[2],
+        _format = map(lambda x:x[2],
                filter(lambda x:x[0].startswith('/dev/') and x[1],
                zip(self.Get('os_location_source'),
                    self.Get('os_location_dest'),
                    self.Get('os_location_perform_format'))))
+        return self.Get('os_addon_disk_perform_format')+_format
 
 class VariableOsInstallDiskId(ReadonlyVariable):
     """
@@ -1277,17 +1436,24 @@ class VariableOsInstallDiskId(ReadonlyVariable):
                           self.Get('os_disk_id')))
         diskId.update(zip(self.Get('cl_autopartition_disk_dev'),
                           [""]*len(self.Get('cl_autopartition_disk_dev'))))
-        def getIdByFS(fs,parttable,oldid):
+        def getIdByFS(fs,parttable,oldid,mp):
             if parttable == "gpt":
-                return PartitionDistributive.formatIdGpt.get(fs,oldid)
+                if mp == '/boot/efi':
+                    fs = 'uefi'
+                fsinfo = FileSystemManager.supportFS.get(fs,{})
+                return fsinfo.get('gpt','8300')
             elif parttable in ("primary","extended","logical"):
-                return PartitionDistributive.formatId.get(fs,oldid)
+                fsinfo = FileSystemManager.supportFS.get(fs,{})
+                return fsinfo.get('msdos','83')
             return oldid
-        return map(lambda x:getIdByFS(x[0],
-                            x[1], diskId.get(x[2],'')) if x[3] else '',
+        FORMAT,PART,DEV,MOUNT,PARENT = 0,1,2,3,4
+        return map(lambda x:getIdByFS(x[FORMAT],
+                            x[PART], diskId.get(x[DEV],''), x[MOUNT] 
+                            ) if x[PARENT] else '',
                self.ZipVars('os_install_disk_format',
                             'os_install_disk_part',
                             'os_install_disk_dev',
+                            'os_install_disk_mount',
                             'os_install_disk_parent'))
 
 class VariableOsInstallDiskName(Variable):
@@ -1316,11 +1482,18 @@ class VariableOsInstallDiskSize(SourceReadonlyVariable):
     indexField = 'os_install_disk_dev'
 
     def getMap(self):
-        return dict(self.ZipVars('os_location_source','os_location_size'))
+        retDict = dict(self.ZipVars('os_disk_dev','os_disk_size'))
+        retDict.update(dict(self.ZipVars('os_location_source','os_location_size')))
+        return retDict
 
     def getMapHumanReadable(self):
-        return dict(zip(self.Get('os_location_source'),
-                            self.Get('os_location_size',humanreadable=True)))
+        retDict = dict(zip(map(getCommonDeviceName,
+                           self.Get('os_disk_dev')),
+                       self.Get('os_disk_size',humanreadable=True)))
+        retDict.update(dict(zip(map(getCommonDeviceName,
+                            self.Get('os_location_source')),
+                            self.Get('os_location_size',humanreadable=True))))
+        return retDict
 
 class VariableOsInstallDiskType(SourceReadonlyVariable):
     """
@@ -1397,6 +1570,25 @@ class VariableOsInstallBindMountpoint(ReadonlyVariable):
         return self.Select('os_location_dest',
                            where='os_location_source',
                            func=lambda x:not x[0].startswith('/dev/') and x[1])
+
+class VariableOsInstallBootloader(ReadonlyVariable):
+    """
+    Bootloader for brief information
+    """
+    def init(self):
+        self.label = _("Bootloader")
+
+    def get(self):
+        if self.Get('os_install_uefi_set') == 'on':
+            return "uefi"
+        else:
+            return ",".join(self.Get('os_install_mbr'))
+
+    def humanReadable(self):
+        if self.Get('os_install_uefi_set') == 'on':
+            return _("UEFI")
+        else:
+            return ",".join(self.Get('os_install_mbr',humanreadable=True))
     
 class VariableOsInstallMbr(LocationHelper,Variable):
     """
@@ -1407,6 +1599,7 @@ class VariableOsInstallMbr(LocationHelper,Variable):
     opt = ["--mbr"]
     metavalue = "MBR"
     untrusted = True
+    check_after = ["os_install_uefi_set"]
 
     def init(self):
         self.label = _("Boot disk")
@@ -1414,6 +1607,8 @@ class VariableOsInstallMbr(LocationHelper,Variable):
 
     def get(self):
         """Get default Master boot record install"""
+        if self.Get('os_install_uefi_set') == 'on':
+            return []
         if self.Get('cl_autopartition_set') == 'on' and \
             self.Get('cl_autopartition_lvm_set') == 'on':
             return self.Get('cl_autopartition_device')
@@ -1497,7 +1692,14 @@ class VariableOsInstallMbr(LocationHelper,Variable):
                       mbrDisk)
             if rootType == "flash":
                 if tableOnBootDisk == "gpt":
-                    raise VariableError(_("You need a disk with a dos table for Flash install"))
+                    raise VariableError(_("You need a disk with a dos "
+                                          "table for Flash install"))
+            if rootType in ("usb-hdd","hdd") and tableOnBootDisk == "gpt":
+                bios_grub = self.Select('os_disk_parent',
+                                        where='os_disk_id',eq='EF02')
+                if not mbrDisk in bios_grub:
+                    raise VariableError(
+                        _("Disk %s must has 'bios_grub' partition")%mbrDisk)
         if value:
             if self.Get('os_grub2_path'):
                 self.checkForGrub2()
@@ -1540,6 +1742,17 @@ class VariableOsInstallMbr(LocationHelper,Variable):
               _("To support booting from %s, legacy grub needs a "
                 "separate /boot partition")%bootDiskFormat)
 
+    def uncompatible(self):
+        """
+        Network setting up unavailable for flash installation
+        """
+        if self.Get('os_install_uefi_set') == "on":
+            return _("MBR is not using with UEFI bootloader")
+        if self.Get('cl_autopartition_set') == "on":
+            return \
+                _("The layout is not available with autopartitioning")
+        return ""
+
 class VariableOsInstallRootType(LocationHelper,Variable):
     """
     Type of installation
@@ -1574,8 +1787,8 @@ class VariableOsInstallRootDev(ReadonlyVariable):
     def get(self):
         """Get install root device"""
         if self.Get('cl_action') == 'system':
-            return self.Select('os_install_disk_dev',
-                               where='os_install_disk_mount',
+            return self.Select('os_install_disk_dev_base',
+                               where='os_install_disk_mount_base',
                                eq="/",limit=1) or ''
         else:
             return self.Get('os_root_dev')
