@@ -24,6 +24,7 @@ from calculate.install.fs_manager import FileSystemManager
 from calculate.lib.utils.files import (readFile,getProgPath,process,
     readLinesFile)
 from calculate.lib.utils.common import getPasswdUsers,getUserGroups,getGroups
+from calculate.lib.utils.common import getValueFromConfig
 from calculate.lib.utils.portage import isPkgInstalled
 from calculate.lib.utils.device import getUdevDeviceInfo
 from crypt import crypt
@@ -73,7 +74,7 @@ class VariableOsInstallScratch(Variable):
             if self.Get('os_install_root_type') == "flash":
                 raise VariableError(
                     _("Flash install does not support the builder mode"))
-            if filter(lambda x: not x in ('/','swap'), 
+            if filter(lambda x: not x in ('/','swap','/boot/efi'), 
                       self.Get('os_install_disk_mount')):
                 raise VariableError(
                     _("The builder mode does not support "
@@ -490,12 +491,12 @@ class VariableOsGrub2Path(Variable):
     """
     def get(self):
         # find grub2-install
-        grubInstall = getProgPath('/sbin/grub2-install')
+        grubInstall = getProgPath('/usr/sbin/grub2-install')
         if grubInstall:
             return grubInstall
         # find grub-install and check, that this is grub2-install (ver 1.99)
-        grubInstall = self.getProgPath('/sbin/grub-install')
-        if grubInstall and filter(lambda x:"1.99" in x,
+        grubInstall = getProgPath('/usr/sbin/grub-install')
+        if grubInstall and filter(lambda x:"1.99" in x or "2.00" in x,
                            process(grubInstall,'-v')):
             return grubInstall
         return ""
@@ -573,3 +574,79 @@ class VariableOsInstallPxePath(Variable):
     def init(self):
         self.label = _("Installation path")
         self.help = _("path for PXE install")
+
+class VariableOsInstallUefiSet(Variable):
+    """
+    Install in UEFI
+    """
+    type  = "bool"
+    opt = ['--uefi']
+
+    def init(self):
+        self.label = _("UEFI boot")
+        self.help = _("use UEFI boot")
+
+    def get(self):
+        if self.Get('cl_autopartition_set') == 'on':
+            return self.Get('cl_autopartition_uefi_set')
+        else:
+            if self.Get('os_install_disk_efi') or \
+                "/boot/efi" in self.Get('os_location_dest'):
+                if self.Get('os_install_arch_machine') == 'x86_64' and \
+                    self.Get('os_install_root_type') != 'flash':
+                    return self.Get('os_uefi_set')
+        return 'off'
+
+    def check(self,value):
+        if value == 'on':
+            if self.Get('os_uefi_set') == 'off' and \
+                self.Get('os_install_root_type') == 'hdd':
+                raise VariableError(
+                  _("System must be loaded in UEFI for using this bootloader"))
+            if not 'gpt' in self.Get('os_device_table'):
+                raise VariableError(
+                    _("Need GPT for using UEFI bootloader"))
+            if not (self.Get('os_install_disk_efi') or \
+                "/boot/efi" in self.Get('os_location_dest')):
+                raise VariableError(
+                    _("Need EF00 partition for using UEFI bootloader"))
+            if self.Get('os_install_arch_machine') != 'x86_64':
+                raise VariableError(
+                    _("Architecture of the target system must be x86_64"))
+            if self.Get('os_install_root_type') == 'flash':
+                raise VariableError(
+                    _("This option not used for Flash install"))
+
+    def uncompatible(self):
+        """
+        Network setting up unavailable for flash installation
+        """
+        if self.Get('cl_autopartition_set') == "on":
+            return \
+                _("The layout is not available with autopartitioning")
+        return ""
+
+class VariableOsInstallGrubTerminal(Variable):
+    """
+    Gfxmode
+    """
+    type  = "choice"
+    opt = ['--grub-terminal']
+    metavalue = "TERMINAL"
+
+    def init(self):
+        self.label = _("Grub terminal")
+        self.help = _("grub terminal")
+
+    def get(self):
+        cmdLine = '/proc/cmdline'
+        if 'grub_nogfxmode' in readFile(cmdLine):
+            return 'console'
+        grubDefault = path.join(self.Get('cl_chroot_path'),
+                                'etc/default/grub')
+        if getValueFromConfig(grubDefault,'GRUB_TERMINAL') == 'console':
+            return 'console'
+        return 'gfxterm'
+
+    def choice(self):
+        return ['gfxterm','console']
